#include <rFlex.h>
#include <rFlexP.h>

/* ============================================================ */
/* ============================================================ */

static int
updateRpl (struct reflex* reflex, unsigned char id,
	   int len, unsigned char *buffer)
{
  int ii;
  int retval;
  int timeStamp;
  int count;

  /* should check on len and opcode */

  ii = 0;

  retval    = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);
  count     = pktGetLong(buffer, &ii);

  if (reflex->dio->rptRpl) {
    retval = reflex->dio->updateRpl(reflex, retval, id, timeStamp, count);
  }

  return(retval);
}

/*------------------------------------------------------------*/

static int
rptRpl (struct reflex* reflex, unsigned char id,
	int len, unsigned char *buffer)
{
  int ii;
  int retval;
  int timeStamp;

  /* should check on len and opcode */

  ii = 0;

  retval    = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);

  if (reflex->dio->rptRpl) {
    retval = reflex->dio->rptRpl(reflex, retval, id, timeStamp);
  }
  return(retval);
}
/*------------------------------------------------------------*/

static int
setRpl (struct reflex* reflex, unsigned char id,
	int len, unsigned char *buffer)
{
  int ii;
  int retval;
  int timeStamp;

  /* should check on len and opcode */

  ii = 0;

  retval    = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);

  fprintf(stderr, __FILE__":%d:"__FUNCTION__
	  "() - retval=%d timeStamp=%d\n",
	  __LINE__, retval, timeStamp);

  if (reflex->dio->setRpl) {
    retval = reflex->dio->rptRpl(reflex, retval, id, timeStamp);
  }
  return(retval);
}
    

/* ============================================================ */

static int
update(struct reflex *reflex, unsigned char id,
       int len, unsigned char *buffer)
{
  int ii;
  int count;
  int timeStamp;
  int retval;
  unsigned char netId, offset, state;

  /* should check on len and opcode */

  ii = 0;
  count = 0;

  timeStamp = pktGetLong(buffer, &ii);
  netId     = pktGetChar(buffer, &ii);
  offset    = pktGetChar(buffer, &ii);
  state     = pktGetChar(buffer, &ii);

  if (reflex->dio->update) {
    retval = reflex->dio->update(reflex, id, timeStamp,
				 netId, offset, state);
  }

  return(retval);
}
  
/*------------------------------------------------------------*/

static int
rpt(struct reflex *reflex, unsigned char id,
    int len, unsigned char *buffer)
{
  int ii;
  int count;
  int timeStamp;
  int retval;
  unsigned char netId, offset, state;

  /* should check on len and opcode */

  ii = 0;
  count = 0;

  timeStamp = pktGetLong(buffer, &ii);
  netId     = pktGetChar(buffer, &ii);
  offset    = pktGetChar(buffer, &ii);
  state     = pktGetChar(buffer, &ii);

  if (reflex->dio->update) {
    retval = reflex->dio->rpt(reflex, id, timeStamp,
			      netId, offset, state);
  }

  return(retval);
}
  
/* ============================================================ */

static int
updateReq(struct reflex *reflex, unsigned char id, int timeStamp)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int retval;

  len = 0;

  opcode = DIO_GET_UPDATE;
  port   = reflex->dio->port;

  pktAddLong(buffer, &len, timeStamp);

  retval = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (retval);
}
  
/*------------------------------------------------------------*/

static int
rptReq(struct reflex *reflex, unsigned char id, int period)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int retval;

  len = 0;

  opcode = DIO_REPORTS_REQ;
  port   = reflex->dio->port;

  pktAddLong(buffer, &len, period);

  retval = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (retval);
}
  
/*------------------------------------------------------------*/

static int
setReq (struct reflex *reflex, unsigned char id,
	unsigned char netId, unsigned char offset,
	unsigned char state, unsigned char mask)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int retval;

  len = 0;

  opcode = DIO_SET;
  port   = reflex->dio->port;

  pktAddChar(buffer, &len, netId);
  pktAddChar(buffer, &len, offset);
  pktAddChar(buffer, &len, state);
  pktAddChar(buffer, &len, mask);

  retval = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (retval);
}
	 
/* ============================================================ */
/* ============================================================ */
/* ============================================================ */

static int
handlePort(struct reflex* reflex,
	   int size, unsigned char *buffer,
	   unsigned char id, unsigned char opcode)
{
  int retval;

  switch (opcode) {
  case DIO_GET_UPDATE:
    retval = updateRpl    (reflex, id, size, buffer);
    break;

  case DIO_REPORTS_REQ:
    retval = rptRpl       (reflex, id, size, buffer);
    break;

  case DIO_UPDATE:
    retval = update       (reflex, id, size, buffer);
    break;

  case DIO_REPORT:
    retval = rpt          (reflex, id, size, buffer);
    break;

  case DIO_SET:
    retval = setRpl       (reflex, id, size, buffer);
    break;

  default:
    retval = 0;
    if (reflex->dio->unknownRpl) {
      retval = reflex->dio->unknownRpl(reflex, id, size, buffer, opcode);
    }
  }

  return(retval);
}

/* ============================================================ */
/* ============================================================ */

static int
destroy (struct reflexDio *dio) {

  if (dio) free(dio);
  return(0);
}

/* ------------------------------------------------------------ */

struct reflexDio *
reflexDioNew (int port) {
  struct reflexDio *dio;

  dio = malloc(sizeof(*dio));
  if (!dio) return(NULL);
  memset(dio, 0, sizeof(*dio));

  dio->port       = port;

  dio->updateReq  = updateReq;
  dio->rptReq     = rptReq;
  dio->setReq     = setReq;

  dio->handlePort = handlePort;
  dio->destroy    = destroy;

  return(dio);
}
