#include <rFlex.h>
#include <rFlexP.h>

/* ============================================================ */

static int readLoop(struct reflex *reflex, int count)
{
  int fd;
  char buffer[256];
  int nread;
  unsigned char port, id, opcode;
  struct pktPort *pkt;
  int rv;

  if (count == 0) {
    return(0);
  }

  pkt = reflex->pktPort;
  fd = pktGetFd(pkt);

  do {
    nread = pktRead(pkt, buffer, &port, &id, &opcode);

    if (nread < 0) {
      if (reflex->handler) {
	rv = reflex->handler (reflex, nread, buffer, port, id, 1);
      }
      continue;
    }

    if (nread > 0) {
      switch (port) {
      case 0:
	if (reflex->handler) {
	  rv = reflex->handler (reflex, nread, buffer, port, id, opcode);
	}
	break;

      case SYS_PORT:
	rv = reflex->system->handlePort (reflex, nread, buffer, id, opcode);
	break;

      case MOT_PORT:
	rv = reflex->motion->handlePort (reflex, nread, buffer, id, opcode);
	break;

      case JSTK_PORT:
	rv = reflex->joystick->handlePort (reflex, nread, buffer, id, opcode);
	break;

      case SONAR_PORT:
	rv = reflex->sonar->handlePort (reflex, nread, buffer, id, opcode);
	break;

      case DIO_PORT:
	rv = reflex->dio->handlePort (reflex, nread, buffer, id, opcode);
	break;

      case IR_PORT:
	rv = reflex->ir->handlePort (reflex, nread, buffer, id, opcode);
	break;

      default:
	if (reflex->handler) {
	  rv = reflex->handler (reflex, nread, buffer, port, id, 2);
	}
      }
    }
    count--;
  } while ((nread != 0) && (count!=0) && (rv != 0));

  return(nread != 0);
}

/* ------------------------------------------------------------ */

static int
destroy (struct reflex *reflex) {

  reflex->system->destroy(reflex->system);
  reflex->motion->destroy(reflex->motion);
  reflex->joystick->destroy(reflex->joystick);
  reflex->sonar->destroy(reflex->sonar);
  reflex->dio->destroy(reflex->dio);
  reflex->ir->destroy(reflex->ir);
  
  free(reflex);
  return(0);
}

/* ------------------------------------------------------------ */

struct reflex *
reflexNew (int fd, void *pArg) {

  struct reflex *reflex;
  reflex = malloc(sizeof *reflex);
  if (!reflex) return(NULL);
  memset(reflex, 0, sizeof(*reflex));

  reflex->system   = reflexSystemNew   (SYS_PORT);
  reflex->motion   = reflexMotionNew   (MOT_PORT);
  reflex->joystick = reflexJoystickNew (JSTK_PORT);
  reflex->sonar    = reflexSonarNew    (SONAR_PORT);
  reflex->dio      = reflexDioNew      (DIO_PORT);
  reflex->ir       = reflexIRNew       (IR_PORT);
  reflex->pArg     = pArg;

  reflex->pktPort  = pktInitPort(fd);

  reflex->read    = readLoop;
  reflex->destroy = destroy;

  return(reflex);
}
