#include <rFlex.h>
#include <rFlexP.h>

/* ============================================================ */
/* ============================================================ */

static int
lcdDumpReq(struct reflex *reflex, unsigned char id)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int retval;

  len = 0;

  opcode = SYS_LCD_DUMP;
  port   = reflex->system->port;

  retval = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (retval);
}

/*------------------------------------------------------------*/

static int
lcdDumpRpl(struct reflex *reflex, unsigned char id,
	   int len, unsigned char *buffer)
{
  int ii;
  int i2;
  int timeStamp;
  unsigned char row, nbytes;
  unsigned char data[256];
  int rv;

  /* should check on len and opcode */

  ii = 0;

  timeStamp = pktGetLong(buffer, &ii);
  row       = pktGetChar(buffer, &ii);
  nbytes    = pktGetChar(buffer, &ii);

  for (i2=0; i2<nbytes; i2++) {
    data[i2] = pktGetChar(buffer, &ii);
  }

  if (reflex->system->lcdDumpRpl) {
    rv = reflex->system->lcdDumpRpl(reflex, rv, id, timeStamp,
				    row, nbytes, data);
  }
  return (rv);
}

/* ============================================================ */

static int
statusReq(struct reflex *reflex, unsigned char id)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int retval;

  len = 0;

  opcode = SYS_STATUS;
  port   = reflex->system->port;

  retval = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (retval);
}

/*------------------------------------------------------------*/

static int
statusRpl(struct reflex *reflex, unsigned char id,
	  int len, unsigned char *buffer)
{
  int ii;
  int ts, volts;
  unsigned char brake;
  int rv;

  /* should check on len and opcode */

  ii = 0;

  ts     = pktGetLong(buffer, &ii);
  volts  = pktGetLong(buffer, &ii);
  brake  = pktGetChar(buffer, &ii);

  if (reflex->system->statusRpl) {
    rv = reflex->system->statusRpl(reflex, rv, id, ts, volts, brake);
  }
  return (rv);
}

/* ============================================================ */
/* ============================================================ */
/* ============================================================ */

static int
handlePort(struct reflex *reflex, int size, unsigned char *buffer,
	   unsigned char id, unsigned char opcode)
{
  int retval;

  switch (opcode) {
  case SYS_LCD_DUMP:
    retval = lcdDumpRpl(reflex, id, size, buffer);
    break;

  case SYS_STATUS:
    retval = statusRpl(reflex, id, size, buffer);
    break;

  default:
    retval = 0;
    if (reflex->system->unknownRpl) {
      retval = reflex->system->unknownRpl(reflex, id, size, buffer, opcode);
    }
  }

  return(retval);
}
/* ============================================================ */
/* ============================================================ */

static int
destroy (struct reflexSystem *ptr) {

  if (ptr) free(ptr);
  return(0);
}

/* ------------------------------------------------------------ */

struct reflexSystem *
reflexSystemNew (int port) {
  struct reflexSystem *sys;

  sys = malloc(sizeof(*sys));
  if (!sys) return(NULL);
  memset(sys, 0, sizeof(*sys));

  sys->port       = port;

  sys->lcdDumpReq  = lcdDumpReq;
  sys->statusReq   = statusReq;

  sys->handlePort = handlePort;
  sys->destroy    = destroy;

  return(sys);
}
